﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Windows;
using Infragistics.Controls.Grids;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;

namespace PivotGridTestApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            pivotGrid.Loaded += FlatDataLoaded;
        }

        private void FlatDataLoaded(object sender, RoutedEventArgs e)
        {
            var flatDataSource = new FlatDataSource
            {
                ItemsSource = CreateCollection(),
                Cube = DataSourceBase.GenerateInitialCube("PivotGridData"),
                Rows = DataSourceBase.GenerateInitialItems("[XAxis].[XAxis]"),
                Columns = DataSourceBase.GenerateInitialItems("[Product].[Product]"),
                Measures = DataSourceBase.GenerateInitialItems("Value")
            };
            pivotGrid.DataSource = flatDataSource;
            dataSelector.DataSource = flatDataSource;
        }

        private IEnumerable CreateCollection()
        {
            var products = new List<PivotGridData>();
            for (var i = 0; i < 100; i++)
            {
                PivotGridData product;
                if(i<10)
                    product = new PivotGridData(i.ToString(), "Product1", 123.12345);
                else if(i<20)
                    product = new PivotGridData(i.ToString(), "Product2", -123.12345);
                else if (i < 20)
                    product = new PivotGridData(i.ToString(), "Product3", 123.12345);
                else if (i < 30)
                    product = new PivotGridData(i.ToString(), "Product4", 123.12345);
                else if (i < 40)
                    product = new PivotGridData(i.ToString(), "Product5", -123.12345);
                else if (i < 50)
                    product = new PivotGridData(i.ToString(), "Product6", 123.12345);
                else if (i < 60)
                    product = new PivotGridData(i.ToString(), "Product7", -123.12345);
                else if (i < 70)
                    product = new PivotGridData(i.ToString(), "Product8", 123.12345);
                else if (i < 80)
                    product = new PivotGridData(i.ToString(), "Product9", -123.12345);
                else if (i < 90)
                    product = new PivotGridData(i.ToString(), "Product10", 123.12345);
                else
                    product = new PivotGridData(i + "Y", "Product11", 123.12345);
                products.Add(product);
            }
            return products;
        }

        private void PivotGridCellControlAttached(object sender, PivotCellControlAttachedEventArgs e)
        {
            e.IsDirty = true;
            if (pivotGrid.DataSource == null) return;

            var rowIndex = pivotGrid.DataRows.IndexOf(e.Cell.DataRow);
            var columnIndex = pivotGrid.DataColumns.IndexOf(e.Cell.DataColumn);
            ICell cellData = pivotGrid.DataSource.Result.Cells[rowIndex, columnIndex];

            if (cellData == null) return;

            var value = 0.0;
            if (!double.TryParse(cellData.Value.ToString(), out value)) return;
            Style s = null;
            if (value < 0)
            s = Resources["RedStyle"] as Style;
            if (e.Cell.Style == null || e.Cell.Style != s)
                e.Cell.Style = s;

            var pivotGridDataSource = pivotGrid.DataSource as FlatDataSource;
            if (pivotGridDataSource == null) return;
            var itemsIndexes = pivotGridDataSource.GetCellItemsIndexes(cellData);
            if (itemsIndexes.Count == 0) return;
            var pivotGridObject = pivotGridDataSource.GetRecord(itemsIndexes[0]);
            if (pivotGridObject == null) return;
            var pivotGridData = pivotGridObject as PivotGridData;
            if (pivotGridData != null)
            {
                var productFormat = GetFormatString(pivotGridData.Product);
                cellData.SetValue(cellData.Value, null, productFormat);
            }
        }

        private string GetFormatString(string product)
        {
            switch (product)
            {
                case "Product1" :
                        return "{0:0.0}";
                case "Product2":
                        return "{0:0.00}";
                case "Product3":
                        return "{0:0.000}";
                case "Product4":
                        return "{0:0.0000}";
                case "Product5":
                        return "{0:0.0%}";
                case "Product6":
                        return "{0:0.00%}";
                case "Product7":
                        return "{0:0.000%}";
                case "Product8":
                        return "{0:0.0000%}";
                case "Product9":
                        return "{0:0}";
                default :
                    return "{0:0}";
            }
        }
    }
}
